aboutgitcodebugslistschat
path: root/tcp_splice.c
diff options
context:
space:
mode:
authorDavid Gibson <david@gibson.dropbear.id.au>2022-11-17 16:58:53 +1100
committerStefano Brivio <sbrivio@redhat.com>2022-11-25 01:35:25 +0100
commit233b95e90f8e9391d93d2187f682eaf51b6f3dd1 (patch)
tree85d80d8cf394f7f905b7354cd29a2eb24ef1523c /tcp_splice.c
parentd909fda1e81979da12ed4ab8b2f2a18fba756a07 (diff)
downloadpasst-233b95e90f8e9391d93d2187f682eaf51b6f3dd1.tar
passt-233b95e90f8e9391d93d2187f682eaf51b6f3dd1.tar.gz
passt-233b95e90f8e9391d93d2187f682eaf51b6f3dd1.tar.bz2
passt-233b95e90f8e9391d93d2187f682eaf51b6f3dd1.tar.lz
passt-233b95e90f8e9391d93d2187f682eaf51b6f3dd1.tar.xz
passt-233b95e90f8e9391d93d2187f682eaf51b6f3dd1.tar.zst
passt-233b95e90f8e9391d93d2187f682eaf51b6f3dd1.zip
tcp: Remove splice from tcp_epoll_ref
Currently the epoll reference for tcp sockets includes a bit indicating whether the socket maps to a spliced connection. However, the reference also has the index of the connection structure which also indicates whether it is spliced. We can therefore avoid the splice bit in the epoll_ref by unifying the first part of the non-spliced and spliced handlers where we look up the connection state. Signed-off-by: David Gibson <david@gibson.dropbear.id.au> Signed-off-by: Stefano Brivio <sbrivio@redhat.com>
Diffstat (limited to 'tcp_splice.c')
-rw-r--r--tcp_splice.c26
1 files changed, 10 insertions, 16 deletions
diff --git a/tcp_splice.c b/tcp_splice.c
index 356d194..bfcf63d 100644
--- a/tcp_splice.c
+++ b/tcp_splice.c
@@ -166,11 +166,9 @@ static int tcp_splice_epoll_ctl(const struct ctx *c,
{
int m = conn->c.in_epoll ? EPOLL_CTL_MOD : EPOLL_CTL_ADD;
union epoll_ref ref_a = { .r.proto = IPPROTO_TCP, .r.s = conn->a,
- .r.p.tcp.tcp.splice = 1,
.r.p.tcp.tcp.index = CONN_IDX(conn),
.r.p.tcp.tcp.v6 = CONN_V6(conn) };
union epoll_ref ref_b = { .r.proto = IPPROTO_TCP, .r.s = conn->b,
- .r.p.tcp.tcp.splice = 1,
.r.p.tcp.tcp.index = CONN_IDX(conn),
.r.p.tcp.tcp.v6 = CONN_V6(conn) };
struct epoll_event ev_a = { .data.u64 = ref_a.u64 };
@@ -547,24 +545,20 @@ bool tcp_splice_conn_from_sock(struct ctx *c, union epoll_ref ref,
}
/**
* tcp_sock_handler_splice() - Handler for socket mapped to spliced connection
* @c: Execution context
- * @ref: epoll reference
+ * @conn: Connection state
+ * @s: Socket fd on which an event has occurred
* @events: epoll events bitmap
*
* #syscalls:pasta splice
*/
-void tcp_sock_handler_splice(struct ctx *c, union epoll_ref ref,
- uint32_t events)
+void tcp_splice_sock_handler(struct ctx *c, struct tcp_splice_conn *conn,
+ int s, uint32_t events)
{
uint8_t lowat_set_flag, lowat_act_flag;
int from, to, *pipes, eof, never_read;
uint32_t *seq_read, *seq_write;
- struct tcp_splice_conn *conn;
-
- assert(!ref.r.p.tcp.tcp.listen);
-
- conn = CONN(ref.r.p.tcp.tcp.index);
if (conn->events == SPLICE_CLOSED)
return;
@@ -580,25 +574,25 @@ void tcp_sock_handler_splice(struct ctx *c, union epoll_ref ref,
}
if (events & EPOLLOUT) {
- if (ref.r.s == conn->a)
+ if (s == conn->a)
conn_event(c, conn, ~A_OUT_WAIT);
else
conn_event(c, conn, ~B_OUT_WAIT);
- tcp_splice_dir(conn, ref.r.s, 1, &from, &to, &pipes);
+ tcp_splice_dir(conn, s, 1, &from, &to, &pipes);
} else {
- tcp_splice_dir(conn, ref.r.s, 0, &from, &to, &pipes);
+ tcp_splice_dir(conn, s, 0, &from, &to, &pipes);
}
if (events & EPOLLRDHUP) {
- if (ref.r.s == conn->a)
+ if (s == conn->a)
conn_event(c, conn, A_FIN_RCVD);
else
conn_event(c, conn, B_FIN_RCVD);
}
if (events & EPOLLHUP) {
- if (ref.r.s == conn->a)
+ if (s == conn->a)
conn_event(c, conn, A_FIN_SENT); /* Fake, but implied */
else
conn_event(c, conn, B_FIN_SENT);
@@ -547,24 +545,20 @@ bool tcp_splice_conn_from_sock(struct ctx *c, union epoll_ref ref,
}
/**
- * tcp_sock_handler_splice() - Handler for socket mapped to spliced connection
+ * tcp_splice_sock_handler() - Handler for socket mapped to spliced connection
* @c: Execution context